#ifndef STATISTICS_HPP_
#define STATISTICS_HPP_

#include <sys/resource.h>
#include <stdint.h>
#include <unistd.h>
#include <vector>
#include <string>

#include "Settings.hpp"
#include "OFStream.hpp"

class ConstraintDB;
class BasicSolver;

/** Class to store all kind of information during the solving process.
 * All Variables and Functions are static in order to make it accessable easily from anywhere inside the program.*/
struct Statistics{

    Statistics(Settings& sets);

    Settings&               settings;

    /**Number of propagations during the solving.*/
    uint64_t                nbr_propagations;

    /**Number of made decisions that were based on the activity of a variable.*/
    uint64_t                nbr_act_decisions;

    /**Number of random decisions.*/
    uint64_t                nbr_rnd_decisions;

    /**Number of restarts during the solving.*/
    uint32_t                nbr_restarts;

    /**The highest number of levels that could be backjumped after a conflict arised.*/
    uint32_t                max_backjumping_distance;

    uint64_t                sum_backjumping;

    /**Counts detailed information about the backjumping. Distances are grouped in classes of 50 steps (1 to 50, 51 to 100, 101 to 150 an so on).*/
    //std::vector<uint32_t>   backjumping_distances;

    /**Number of propagations where the WatchedLiteral data-structure worked.*/
    //uint32_t                nbr_propagate_shortcut_failed;

    /**Number of propagations where the WatchedLiteral data-structure did not work.*/
    //uint32_t                nbr_propagate_shortcut_worked;

    /**Total number of steps that had to be made to find new watchers.*/
    //uint64_t                nbr_search_steps_for_watcher;

    /**Number of clause bumps where decaying became expensive.*/
    uint32_t                nbr_expensive_cls_bmp;

    /**Number of var bumps where decaying became expensive.*/
    uint32_t                nbr_expensive_var_bmp;

    uint32_t 				nbr_flipped_prop_learnts;

    /**Number of calls to reduce the amount of learnt clauses.*/
    uint32_t                nbr_learnt_reductions;

    uint64_t                total_lemma_shortening;

    uint32_t                nbr_simplify_execs;

    uint32_t                nbr_removed_cls_lits;

    uint32_t                nbr_shrink_cls_lits;

    uint32_t                nbr_simplify_worked;

    uint32_t 				nbr_bigsimplify_shrinked;

    uint32_t                max_dec_level;

    uint32_t                nbr_prob_cls_decisions;

    uint32_t                nbr_expensive_cls_stamp;

    uint32_t				nbr_dmrp_calls;

    /**number of blocking clauses*/
	uint32_t				nbr_block_clauses;

	/**number of clauses in original formula*/
	uint32_t				nbr_clauses_original;

	/**number of literals in original formula*/
	uint32_t				nbr_literals_original;

	/**number of clauses in simplified formula*/
	 uint32_t				nbr_clauses_simplified;

	 /**number of literals in simplified formula*/
	 uint32_t				nbr_literals_simplified;

	/**Timestamp after parsing the daimler formula.*/
	double                  parse_timestmp;

	/**Timestamp after simplifying the formula.*/
	double					simplifying_timestmp;

    /**Timestamp after solving of the converted formula.*/
    double                  finish_timestmp;

    /**Logs the backjumping distance for the detailed statistics.
     * @param dist: the number of levels that were skipped due to backjumping*/
    void                    storeBackjumpingDistance(const int dist);

    /**Print all statistic information to the given OFStream.
     * @param out: the stream to printDescription to*/
    void                    printFinalStatistics(OFStream& out, const BasicSolver* sp );

    /**Print all statistic information to the given OFStream.
    * @param out: the stream to printDescription to*/
    void 					printStatsProgress(const char c,uint64_t literals,uint64_t nbr_conflicts);


    /**Print all statistic information to the given OFStream.
    * @param out: the stream to printDescription to*/
    void                    printStatsProgressHead();



    /**Print all statistic information into a comma seperated file.
     * Instead of a seperation by commata the character '&' is used to avoid clashes with decimal commata for local printing.
    * @param out: the stream to printDescription to
    * @param result: the result of the solving process (SAT/UNSAT/Trivial UNSAT ...)*/
    void                    printCSV(OFStream& out,const BasicSolver* sp, char* result);

    /**Get the system time that was used by the cpu for this process.
     * @return the system time in seconds*/
    double                  cpuSystemTime();

    /**Get the user time that was used by the cpu for this process.
     * @return the user time in seconds.*/
    double                  cpuUserTime();

    /**Get the amount of bytes that were used by this process.
     * @return the amount of bytes (in bytes)*/
    uint64_t                usedMemoryBytes();


protected:
    void                    printFile(OFStream& out,const char* seperator, const BasicSolver* sp, char* result);

};

//>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>Implementation








const char*  statistics_cpp_version();
#define statistics_hpp_version "Statistics.hpp:     $Revision: 1.1 $"
#endif /*STATISTICS_HPP_*/
